package com.isoft.common.sm4;

import lombok.extern.slf4j.Slf4j;
import sun.misc.BASE64Decoder;
import sun.misc.BASE64Encoder;

import java.io.IOException;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

@Slf4j
public class SM4Utils {

    private final String secretKey = "FFFAAA333666DDDB";
    private String iv = "";
    private  boolean hexString = false;

    public SM4Utils() {
        //
         }

    public String encryptData_ECB(String plainText)
    {
        try
        {
            SM4_Context ctx = new SM4_Context();
            ctx.isPadding = true;
            ctx.mode = SM4.SM4_ENCRYPT;

            byte[] keyBytes;
            if (hexString)
            {
                keyBytes = Util.hexStringToBytes(secretKey);
            }
            else
            {
                keyBytes = secretKey.getBytes();
            }

            SM4 sm4 = new SM4();
            sm4.sm4_setkey_enc(ctx, keyBytes);
            byte[] encrypted = sm4.sm4_crypt_ecb(ctx, plainText.getBytes("GBK"));
            String cipherText = new BASE64Encoder().encode(encrypted);
            if (cipherText != null && cipherText.trim().length() > 0)
            {
                Pattern p = Pattern.compile("\\s*|\t|\r|\n");
                Matcher m = p.matcher(cipherText);
                cipherText = m.replaceAll("");
            }
            return cipherText;
        }
        catch (Exception e)
        {
            log.error(e.getMessage());
            return null;
        }
    }

    public String decryptData_ECB(String cipherText)
    {
        try
        {
            SM4_Context ctx = new SM4_Context();
            ctx.isPadding = true;
            ctx.mode = SM4.SM4_DECRYPT;

            byte[] keyBytes;
            if (hexString)
            {
                keyBytes = Util.hexStringToBytes(secretKey);
            }
            else
            {
                keyBytes = secretKey.getBytes();
            }

            SM4 sm4 = new SM4();
            sm4.sm4_setkey_dec(ctx, keyBytes);
            byte[] decrypted = sm4.sm4_crypt_ecb(ctx, new BASE64Decoder().decodeBuffer(cipherText));
            return new String(decrypted, "GBK");
        }
        catch (Exception e)
        {
            log.error(e.getMessage());
            return null;
        }
    }

    public String encryptData_CBC(String plainText)
    {
        try
        {
            SM4_Context ctx = new SM4_Context();
            ctx.isPadding = true;
            ctx.mode = SM4.SM4_ENCRYPT;

            byte[] keyBytes;
            byte[] ivBytes;
            if (hexString)
            {
                keyBytes = Util.hexStringToBytes(secretKey);
                ivBytes = Util.hexStringToBytes(iv);
            }
            else
            {
                keyBytes = secretKey.getBytes();
                ivBytes = iv.getBytes();
            }

            SM4 sm4 = new SM4();
            sm4.sm4_setkey_enc(ctx, keyBytes);
            byte[] encrypted = sm4.sm4_crypt_cbc(ctx, ivBytes, plainText.getBytes("GBK"));
            String cipherText = new BASE64Encoder().encode(encrypted);
            if (cipherText != null && cipherText.trim().length() > 0)
            {
                Pattern p = Pattern.compile("\\s*|\t|\r|\n");
                Matcher m = p.matcher(cipherText);
                cipherText = m.replaceAll("");
            }
            return cipherText;
        }
        catch (Exception e)
        {
            log.error(e.getMessage());
            return null;
        }
    }

    public String decryptData_CBC(String cipherText)
    {
        try
        {
            SM4_Context ctx = new SM4_Context();
            ctx.isPadding = true;
            ctx.mode = SM4.SM4_DECRYPT;

            byte[] keyBytes;
            byte[] ivBytes;
            if (hexString)
            {
                keyBytes = Util.hexStringToBytes(secretKey);
                ivBytes = Util.hexStringToBytes(iv);
            }
            else
            {
                keyBytes = secretKey.getBytes();
                ivBytes = iv.getBytes();
            }

            SM4 sm4 = new SM4();
            sm4.sm4_setkey_dec(ctx, keyBytes);
            byte[] decrypted = sm4.sm4_crypt_cbc(ctx, ivBytes, new BASE64Decoder().decodeBuffer(cipherText));
            return new String(decrypted, "GBK");
        }
        catch (Exception e)
        {
           log.error(e.getMessage());
            return null;
        }
    }

    public static void main(String[] args) throws IOException
    {
        String plainText = "{\"username\":\"黄龙\",\"password\":\"admin123\",\"code\":\"bnkh\",\"uuid\":\"0a12b0e284ee496ab034b84f0bd93140\"}";
        String text = "TuR5AnzeCITowfTgv9NbYc9ma39WdsJb5WeQxvQdOzSo/OR3IpJsDvkdOtPa4FgHwC4M3GHrFYo9IYY7ZrdK%2BwA84zVEZj8MAqVTs5H9ce93OyP8g%2B8FtoTM1buS%2Bv3EYAiCRpU0JdK42%2B8LOA3fYr/biTTwPU1TZDW7PMaJ1UL2aVSs%2B0sK6zjnQbY6WxbloGi3%2BN3tuhf%2BJvJfyy%2BEG9Asr6z4E/00Z%2BS8K8xtRYK7olgPCAgaj2evr7AAb3iRyh5Lwx1qnx4msxx5vWj9NzPf8WHjBYqCfTozMRdmWas=";
        SM4Utils sm4 = new SM4Utils();

        System.out.println("ECB模式加密");
        String cipherText = sm4.encryptData_ECB(plainText);
        System.out.println("密文: " + cipherText);
        System.out.println("");

        plainText = sm4.decryptData_ECB("IYScgT2hKFNzP2uxSG6+if3bq0f8U0Ozft49cX7Nw6EEeJZ3eInBxd5+pfxXBXQA2WdDay4lXL0sepSik4PQdZMtdFSPW9kGHycWOUd+MXqkzVaxQxf+8qSfR6vgvZlCaeuvZWrVAqowzK67yXgTsw==");
        System.out.println("明文: " + plainText);
        System.out.println("");

//        System.out.println("CBC模式加密");
//        sm4.iv = "UISwD9fW6cFh9SNS";
//        String cipherText = sm4.encryptData_CBC(plainText);
//        System.out.println("密文: " + cipherText);
//        System.out.println("");
//
//        System.out.println("CBC模式解密");
//        plainText = sm4.decryptData_CBC(cipherText);
//        System.out.println("明文: " + plainText);

//        System.out.println("密文：4esGgDn/snKraRDe6uM0jQ==");
//        String cipherText2 = "4esGgDn/snKraRDe6uM0jQ==";
//        plainText = sm4.decryptData_CBC(cipherText2);
//        System.out.println("明文: " + plainText);
    }

}
