package com.isoft.framework.security.handle;

import java.io.IOException;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.isoft.framework.redis.RedisCache;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.core.Authentication;
import org.springframework.security.web.authentication.logout.LogoutSuccessHandler;
import com.alibaba.fastjson.JSON;
import com.isoft.common.constant.Constants;
import com.isoft.common.constant.HttpStatus;
import com.isoft.common.utils.ServletUtils;
import com.isoft.common.utils.StringUtils;
import com.isoft.framework.manager.AsyncManager;
import com.isoft.framework.manager.factory.AsyncFactory;
import com.isoft.framework.security.LoginUser;
import com.isoft.framework.security.service.TokenService;
import com.isoft.framework.web.domain.AjaxResult;

/**
 * 自定义退出处理类 返回成功
 * 
 * @author isoft
 */
@Configuration
public class LogoutSuccessHandlerImpl implements LogoutSuccessHandler
{
    @Autowired
    private TokenService tokenService;
    @Autowired
    private RedisCache redisCache;

    /**
     * 退出处理
     * 
     * @return
     */
    @Override
    public void onLogoutSuccess(HttpServletRequest request, HttpServletResponse response, Authentication authentication)
            throws IOException, ServletException
    {
        LoginUser loginUser = tokenService.getLoginUser(request);
        if (StringUtils.isNotNull(loginUser))
        {
            String userName = loginUser.getUsername();
            // 删除用户缓存记录
            tokenService.delLoginUser(loginUser.getToken());
            //清除用户 菜单、权限、角色 缓存
            Long userId = loginUser.getUser().getUserId();
            redisCache.deleteObject("ROLES:USER:"+userId);
            redisCache.deleteObject("PERMISSIONS:USER:"+userId);
            redisCache.deleteObject("MENUS:USER:" + userId + "*");

            // 记录用户退出日志
            AsyncManager.me().execute(AsyncFactory.recordLogininfor(userName, Constants.LOGOUT, "退出成功"));
        }
        ServletUtils.renderString(response, JSON.toJSONString(AjaxResult.error(HttpStatus.SUCCESS, "退出成功")));
    }
}
