package com.isoft.framework.security.service;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Service;
import com.isoft.common.enums.UserStatus;
import com.isoft.common.exception.BaseException;
import com.isoft.common.utils.StringUtils;
import com.isoft.framework.security.LoginUser;
import com.isoft.project.system.domain.SysUser;
//import com.isoft.project.system.service.ISysUserService;

/**
 * 用户验证处理
 *
 * @author isoft
 */
@Service
public class UserDetailsServiceImpl implements UserDetailsService
{
    private static final Logger log = LoggerFactory.getLogger(UserDetailsServiceImpl.class);

//    @Autowired
//    private ISysUserService userService;

//    @Autowired
//    private SysPermissionService permissionService;

    @Override
    public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException
    {
//        SysUser user = userService.selectUserByUserName(username);
        SysUser user = null;
        if (StringUtils.isNull(user))
        {
            log.info("登录用户：{} 不存在.", username);
            throw new UsernameNotFoundException("登录用户：" + username + " 不存在");
        }
        else if (UserStatus.DELETED.getCode().equals(user.getDelFlag()))
        {
            log.info("登录用户：{} 已被删除.", username);
            throw new BaseException("对不起，您的账号：" + username + " 已被删除");
        }
        else if (UserStatus.NOT_ALLOW_lOGIN.getCode().equals(user.getAllowLogin()))
        {
            log.info("登录用户：{} 已被禁止登录.", username);
            throw new BaseException("您的账号已被禁止登录，请联系管理员");
        }
        else if (StringUtils.isNull(user.getDeptId()))
        {
            log.info("登录用户：{} 没有部门.", username);
            throw new BaseException("您的账号未分配部门，请联系管理员");
        }

        return createLoginUser(user);
    }

    public UserDetails createLoginUser(SysUser user)
    {
//        return new LoginUser(user, permissionService.getMenuPermission(user));
        return new LoginUser(user, null);
    }
}
