package com.isoft.project.common;

import com.alibaba.fastjson.JSON;
import com.isoft.common.constant.Constants;
import com.isoft.common.exception.CustomException;
import com.isoft.common.utils.DateUtils;
import com.isoft.common.utils.StringUtils;
import com.isoft.common.utils.file.FileUploadUtils;
import com.isoft.common.utils.file.FileUtils;
import com.isoft.framework.config.IsoftConfig;
import com.isoft.framework.config.ServerConfig;
import com.isoft.framework.redis.RedisCache;
import com.isoft.framework.security.filter.JwtAuthenticationTokenFilter;
import com.isoft.framework.security.service.TokenService;
import com.isoft.framework.web.domain.AjaxResult;
import com.isoft.project.util.CheckRegUtil;
import io.jsonwebtoken.Claims;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import jdk.nashorn.internal.runtime.JSONFunctions;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang.StringEscapeUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLConnection;
import java.nio.charset.Charset;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.*;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;


/**
 * 通用请求处理
 * 
 * @author isoft
 */
@Api(tags = "上传接口")
@RestController
public class CommonController
{
    private static final Logger log = LoggerFactory.getLogger(CommonController.class);

    @Autowired
    private ServerConfig serverConfig;

    @Autowired
    RedisCache redisCache;

    @Autowired
    private TokenService tokenService;

    /**
     * token认证过滤器
     */
    @Autowired
    private JwtAuthenticationTokenFilter authenticationTokenFilter;


    /**
     * 通用上传请求
     */
    @ApiOperation(value = "单文件上传", notes = "单文件上传")
    @PostMapping("/common/upload")
    public AjaxResult uploadFile(MultipartFile file , String token) throws Exception {

        if (StringUtils.isNull(token)) {
            return AjaxResult.success("token为空");
        }
        if (StringUtils.isNull(file)) {
            return AjaxResult.success("请选择文件");
        }
        Claims claims = tokenService.parseToken(token);
        Object redisToken = redisCache.getCacheObject("login_tokens:" + claims.get("login_user_key").toString());
        if (StringUtils.isNull(redisToken)){
            return AjaxResult.error("用户未登录");
        }
        try {
            // 上传文件路径
            String filePath = IsoftConfig.getUploadPath();
            // 上传并返回新文件名称
            String fileName = FileUploadUtils.upload(filePath, file,"1");
//
//            //获取文件名称及后缀
//            String originalFilename = file.getOriginalFilename();
//            //获取最后一个.的位置
//            int lastIndexOf = originalFilename.lastIndexOf(".");
//            //获取文件的后缀名 .jpg
//            String suffix = originalFilename.substring(lastIndexOf);
//            //根据文件上传的路径转成pdf, 再次上传
//            String pdfUrl1 =null;
//            if (suffix.equals(".doc")||suffix.equals(".docx")){
//                //word上传后的路径
//                String wordUrl = filePath +File.separator + fileName;
//                //pdf存储路径
//                String[] split = fileName.split("/");
//                String substring2 = split[3].substring(0, 32);
//                String pdfUrl = filePath+File.separator+
//                         split[0]+File.separator+
//                         split[1]+File.separator+
//                         split[2]+File.separator+
//                         substring2 + ".pdf";
//                int i = new WordToPdf().transformToPdf(wordUrl, pdfUrl);
//            }

            HashMap<Object, Object> map = new HashMap<>();
            map.put("msg",fileName);
            map.put("size",file.getSize());
            map.put("name",file.getOriginalFilename());
            return AjaxResult.success(map);
        } catch (Exception e) {
            return AjaxResult.error(e.getMessage());
        }
    }

    /**
     * 通用上传请求
     */
    @ApiOperation(value = "多文件上传", notes = "多文件上传")
//    @Log(title = "多文件上传", businessType = BusinessType.OTHER)
    @PostMapping("/common/uploadList")
    public AjaxResult uploadFileList(@RequestParam("files") List<MultipartFile> files ,@RequestParam("token") String token) {

        if (StringUtils.isNull(token)) {
            return AjaxResult.success("token为空");
        }
        if (StringUtils.isNull(files)) {
            return AjaxResult.success("请选择文件");
        }
        Claims claims = tokenService.parseToken(token);
        Object redisToken = redisCache.getCacheObject("login_tokens:" + claims.get("login_user_key").toString());
        if (StringUtils.isNull(redisToken)){
            return AjaxResult.error("用户未登录");
        }
        List<Map<String,Object>> mapList = new ArrayList<>();
        Map<String,Object> map = null;
        String fileName = null;
        try {
            for (MultipartFile file : files) {
                // 上传文件路径
                String filePath = IsoftConfig.getUploadPath();
                // 上传并返回新文件名称
                fileName = FileUploadUtils.upload(filePath, file,"1");

                //获取文件名称及后缀
                String originalFilename = file.getOriginalFilename();
                //获取最后一个.的位置
                int lastIndexOf = originalFilename.lastIndexOf(".");
                //获取文件的后缀名 .jpg
                String suffix = originalFilename.substring(lastIndexOf);
                //根据文件上传的路径转成pdf, 再次上传
                String pdfUrl1 =null;
                if (suffix.equals(".doc")||suffix.equals(".docx")){
                    //word上传后的路径
                    String wordUrl = filePath +File.separator + fileName;
                    //pdf存储路径
                    String[] split = fileName.split("/");
                    String substring2 = split[3].substring(0, 32);
                    String pdfUrl = filePath+File.separator+
                             split[0]+File.separator+
                             split[1]+File.separator+
                             split[2]+File.separator+
                             substring2 + ".pdf";
                    CheckRegUtil.doc2pdf(wordUrl, pdfUrl);
//                    int i = new WordToPdf().transformToPdf(wordUrl, pdfUrl);
                }

                map = new HashMap<>();
                map.put("fileName", fileName);
                map.put("size",file.getSize());
                map.put("name",file.getOriginalFilename());
                mapList.add(map);
            }
            return AjaxResult.success(mapList);
        } catch (Exception e) {
            return AjaxResult.error(e.getMessage());
        }
    }


    /**
     * 压缩文件上传并解压
     */
    @ApiOperation(value = "压缩文件上传并解压", notes = "压缩文件上传并解压")
    @PostMapping("/common/uploadZip")
    public AjaxResult uploadZip(MultipartFile multipartFile , String token) throws Exception {

        long start = System.currentTimeMillis();
        if (StringUtils.isNull(token)) {
            return AjaxResult.success("token为空");
        }

        int pos = multipartFile.getOriginalFilename().lastIndexOf(".");
        String st = multipartFile.getOriginalFilename().substring(pos + 1).toLowerCase();
        if (StringUtils.isNull(multipartFile)) {
            return AjaxResult.error("压缩文件不能为空");
        }else if (!st.equals("zip")){
            return AjaxResult.error("上传文件格式错误，请重新上传");
        }

        Claims claims = tokenService.parseToken(token);
        Object redisToken = redisCache.getCacheObject("login_tokens:" + claims.get("login_user_key").toString());
        if (StringUtils.isNull(redisToken)){
            return AjaxResult.error("用户未登录");
        }
        // 上传文件路径
        String filePath = IsoftConfig.getUploadPath();
        File toFile = null;
        String fileName = multipartFile.getOriginalFilename();
        List<Map<String,Object>> mapList = new ArrayList<>();
        if (null != fileName && !"".equals(fileName)) {
            toFile = new File(filePath + File.separator + multipartFile.getOriginalFilename());
            try (InputStream ins = multipartFile.getInputStream();
                 OutputStream os = new FileOutputStream(toFile);) {
                //将文件读到服务器中
                byte[] buf = new byte[1024];
                int len;
                while ((len = ins.read(buf)) > 0) {
                    os.write(buf, 0, len);
                }
                os.close();
                ins.close();

                try (ZipFile zip = new ZipFile(toFile, Charset.forName("GBK"))) {
                    //解压服务器文件
                    for (Enumeration entries = zip.entries(); entries.hasMoreElements(); ) {
                        Map<String,Object> map = null;
                        //获取文件
                        ZipEntry entry = (ZipEntry) entries.nextElement();
                        //获取文件名
                        String zipEntryName = entry.getName();
                        String outPath = (filePath + File.separator + DateUtils.datePath() +
                                File.separator + zipEntryName).replaceAll("\\*", "/");
                        // 判断路径是否存在,不存在则创建文件路径
                        File fileDirectory = new File(outPath.substring(0, outPath.lastIndexOf('/')));
                        if (!fileDirectory.exists()) {
                            fileDirectory.mkdirs();
                        }
                        String[] str = zipEntryName.split("/");
                        // 判断文件全路径是否为文件夹,如果是上面已经上传,不需要解压
                        if (new File(outPath).isDirectory()) {
                            delDir(filePath + File.separator + DateUtils.datePath() +
                                    File.separator + str[0]);
                            continue;
                        }

                        delDir(filePath + File.separator + DateUtils.datePath() +
                                File.separator + str[0]);


                        //生成唯一的Md5值
                        String s = DigestUtils.md5Hex(zip.getInputStream(entry));


                        //获取文件后缀
                        String extension = zipEntryName.substring(zipEntryName.lastIndexOf("."));
                        String fileNamePath = filePath +  File.separator  + DateUtils.datePath() + File.separator
                                + s+extension;

                        String newUrl = DateUtils.datePath() + File.separator  + s+extension;

                        try (InputStream in = zip.getInputStream(entry);
                             OutputStream  out = new FileOutputStream(fileNamePath);){
                            byte[] buf1 = new byte[1024];
                            int len1;
                            while ((len1 = in.read(buf1)) > 0) {
                                out.write(buf1, 0, len1);
                            }
                            out.close();
                            in.close();
                        } catch (Exception e){
                            log.error("文件流错误" , e);
                            throw new CustomException("读取文件失败");
                        }
                        map = new HashMap<>();
                        map.put("oldUrl",zipEntryName);
                        map.put("newUrl",newUrl);
                        map.put("fileSize",entry.getSize());
                        mapList.add(map);

                    }
                }catch (Exception e){
                    log.error("上传文件格式错误，请重新上传" , e);
                    throw new CustomException("上传文件格式错误，请重新上传");

                }
            }
        }

        return AjaxResult.success(mapList);
    }



    /**
     * 单文件明文上传请求
     */
    @ApiOperation(value = "单文件明文上传", notes = "单文件明文上传")
    @PostMapping("/common/uploadPlaintextFile")
    public AjaxResult uploadPlaintextFile(MultipartFile file , String token) throws Exception {

        if (StringUtils.isNull(token)) {
            return AjaxResult.success("token为空");
        }
        if (StringUtils.isNull(file)) {
            return AjaxResult.success("请选择文件");
        }
        Claims claims = tokenService.parseToken(token);
        Object redisToken = redisCache.getCacheObject("login_tokens:" + claims.get("login_user_key").toString());
        if (StringUtils.isNull(redisToken)){
            return AjaxResult.error("用户未登录");
        }
        try {
            // 上传文件路径
            String filePath = IsoftConfig.getUploadPath();
            // 上传并返回新文件名称
            String fileName = FileUploadUtils.upload(filePath, file,"0");
            HashMap<Object, Object> map = new HashMap<>();
            map.put("msg",fileName);
            map.put("size",file.getSize());
            map.put("name",file.getOriginalFilename());
            return AjaxResult.success(map);
        } catch (Exception e) {
            return AjaxResult.error(e.getMessage());
        }
    }



    /**
     * 单文件无验证上传
     */
    @ApiOperation(value = "单文件无验证上传", notes = "单文件无验证上传")
    @PostMapping("/common/uploadFileNoToken")
    public AjaxResult uploadFileNoToken(MultipartFile file,String token) throws Exception {
        if (StringUtils.isNull(file)) {
            return AjaxResult.success("请选择文件");
        }
        try {
            // 上传文件路径
            String filePath = IsoftConfig.getUploadPath();
            // 上传并返回新文件名称
            String fileName = FileUploadUtils.upload(filePath, file,"1");
            HashMap<Object, Object> map = new HashMap<>();
            map.put("msg",fileName);
            map.put("size",file.getSize());
            map.put("name",file.getOriginalFilename());
            return AjaxResult.success(map);
        } catch (Exception e) {
            throw new CustomException(e.getMessage());
//            return AjaxResult.error(e.getMessage());
        }
    }






    /**
     * 删除文件或文件夹以及文件夹下所有文件
     * @param dirPath 文件路径
     */
    public static void delDir(String dirPath) {
        try {
            File dirFile = new File(dirPath);
            if (!dirFile.exists()) {
                return;
            }
            if (dirFile.isFile()) {
                Files.delete(Paths.get(dirPath));
                return;
            }
            File[] files = dirFile.listFiles();
            if(files==null){
                return;
            }
            for (File file : files) {
                delDir(file.toString());
            }
            Files.delete(Paths.get(dirPath));
        } catch (IOException e) {
            log.error("文件删除错误" , e);
        }
    }

    /**
     * 通用下载请求
     *
     * @param fileName 文件名称
     * @param delete 是否删除
     */
//    @Log(title = "业务中台-通用请求管理-下载", businessType = BusinessType.OTHER)
    @ApiOperation(value = "业务中台-通用请求管理-下载",notes = "")
    @GetMapping("common/download")
    public void fileDownload(String fileName, Boolean delete, HttpServletResponse response, HttpServletRequest request)
    {
        try
        {
            if (!FileUtils.isValidFilename(fileName))
            {
                throw new Exception(StringUtils.format("文件名称({})非法，不允许下载。 ", fileName));
            }
            String realFileName = System.currentTimeMillis() + fileName.substring(fileName.indexOf("_") + 1);
//            String realFileName = System.currentTimeMillis() + fileName.substring(fileName.indexOf("_") + 1);
            String filePath = IsoftConfig.getDownloadPath() + fileName;

            response.setCharacterEncoding("utf-8");
            response.setContentType("multipart/form-data");
            response.setHeader("Content-Disposition",
                    "attachment;fileName=" + FileUtils.setFileDownloadHeader(request, fileName));
            FileUtils.writeBytes(filePath, response.getOutputStream());
            if (delete)
            {
                FileUtils.deleteFile(filePath);
            }
        }
        catch (Exception e)
        {
            log.error("下载文件失败", e);
        }
    }


    /**
     * 本地资源通用下载
     */
//    @ApiOperation(value = "本地资源通用下载", notes = "本地资源通用下载")
//    @Log(title = "业务中台-通用管理-下载", businessType = BusinessType.OTHER)
    @GetMapping("/common/download/resource")
    public void resourceDownload(String name, HttpServletRequest request, HttpServletResponse response) throws Exception
    {
        // 本地资源路径
        String localPath = IsoftConfig.getProfile();
        // 数据库资源地址
        String downloadPath = localPath + StringUtils.substringAfter(name, Constants.RESOURCE_PREFIX);
        // 下载名称
        String downloadName = StringUtils.substringAfterLast(downloadPath, "/");
        response.setCharacterEncoding("utf-8");
        response.setContentType("multipart/form-data");
        response.setHeader("Content-Disposition",
                "attachment;fileName=" + FileUtils.setFileDownloadHeader(request, downloadName));
        FileUtils.writeBytes(downloadPath, response.getOutputStream());
    }



    /**
     *
     * @param urlPath     下载路径
     * @param fileSavePath 下载存放目录,包含文件名
     * @return 返回下载文件
     * @throws Exception
     */
    public static File downloadFile(String urlPath, String fileSavePath) throws Exception {

        log.info("进入下载文件工具类");
        File file = null;
        BufferedInputStream bin = null;
        OutputStream out = null;
        try {
            // 统一资源
            URL url = new URL(urlPath);
            // 连接类的父类，抽象类
            URLConnection urlConnection = url.openConnection();
            // http的连接类
            HttpURLConnection httpURLConnection = (HttpURLConnection) urlConnection;
            // 设定请求的方法，默认是GET
            httpURLConnection.setRequestMethod("GET");
            // 设置字符编码
            httpURLConnection.setRequestProperty("Charset", "UTF-8");
            // 打开到此 URL 引用的资源的通信链接（如果尚未建立这样的连接）。
            httpURLConnection.connect();

            // 文件大小
            int fileLength = httpURLConnection.getContentLength();

            // 文件名
            String filePathUrl = httpURLConnection.getURL().getFile();
            String fileFullName = filePathUrl.substring(filePathUrl.lastIndexOf(File.separatorChar) + 1);
            fileFullName = fileFullName.substring(fileFullName.lastIndexOf("/") + 1);
            log.info("开始下载文件：" + fileFullName);
            log.info("file length---->" + fileLength);

            url.openConnection();

            bin = new BufferedInputStream(httpURLConnection.getInputStream());

            String path = fileSavePath;
            file = new File(path);
            if (!file.getParentFile().exists()) {
                file.getParentFile().mkdirs();
            }
            out = new FileOutputStream(file);
            int size = 0;
            int len = 0;
            byte[] buf = new byte[1024];
            while ((size = bin.read(buf)) != -1) {
                len += size;
                out.write(buf, 0, size);
                // 打印下载百分比
                if ((len * 100 / fileLength)>50&&(len * 100 / fileLength)<55) {
                    log.info("下载了-------> " + len * 100 / fileLength + "%");
                }else    if ((len * 100 / fileLength)>=98) {
                    log.info("下载了-------> " + len * 100 / fileLength + "%");
                }
                //logger.info("下载了-------> " + len * 100 / fileLength + "%");
            }
            return file;
        } catch (Exception e) {
            // TODO Auto-generated catch block
            throw new CustomException(e.toString()+"文件下载异常，请检查下载链接$$"+urlPath);
        } finally {
            if (bin!=null) {
                bin.close();
            }
            if (out!=null) {
                out.close();
            }
        }
    }







    /**
     * @MethodName: deleteFile
     * @Description: 计划任务的入口，每天凌晨4点，删除前一天文件夹
     * @author: HaoChangXi
     * @date 2021/3/26
     */
    public void deleteFile() {
        Calendar calendar = Calendar.getInstance();
        // 获取当前年
        int year = calendar.get(Calendar.YEAR);
        // 获取当前月
        int month = calendar.get(Calendar.MONTH) + 1;
        // 获取当前日的前一日
        int day = calendar.get(Calendar.DATE) -2;

        //获取文件夹路径
        String uploadPath = IsoftConfig.getUploadPath();
        String dirPath = uploadPath+ File.separator + year + File.separator + month + File.separator + day;
        //删除文件或文件夹以及文件夹下所有文件
        delDir2(dirPath);
    }


    /**
     * 删除文件或文件夹以及文件夹下所有文件
     * @param dirPath 文件路径
     */
    public void delDir2(String dirPath) {
        try {
            File dirFile = new File(dirPath);
            if (!dirFile.exists()) {
                return;
            }
            if (dirFile.isFile()) {
                Files.delete(Paths.get(dirPath));
                return;
            }
            File[] files = dirFile.listFiles();
            if(files==null){
                return;
            }
            for (File file : files) {
                delDir2(file.toString());
            }
            Files.delete(Paths.get(dirPath));
        } catch (IOException e) {
            throw  new CustomException("文件删除异常");
        }

    }


}
