package com.isoft.project.monitor.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.isoft.common.enums.SequenceEnum;
import com.isoft.common.utils.DateUtils;
import com.isoft.common.utils.Threads;
import com.isoft.project.monitor.domain.SysOperLog;
import com.isoft.project.monitor.dto.SysLogininforReqDTO;
import com.isoft.project.monitor.mapper.SysLogininforCommonMapper;
import com.isoft.project.monitor.mapper.SysOperLogCommonMapper;
//import com.isoft.project.tool.TransactionalService;
import jdk.nashorn.internal.runtime.logging.Logger;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import com.isoft.project.monitor.domain.SysLogininfor;
import com.isoft.project.monitor.mapper.SysLogininforMapper;
import com.isoft.project.monitor.service.ISysLogininforService;

/**
 * 系统访问日志情况信息 服务层处理
 *
 * @author isoft
 */
@Slf4j
@Service
@Logger
public class SysLogininforServiceImpl implements ISysLogininforService {

    private static final org.slf4j.Logger logger = LoggerFactory.getLogger(Threads.class);
    @Autowired
    private SysLogininforMapper logininforMapper;
    @Autowired
    private SysOperLogCommonMapper operLogCommonMapper;
    @Autowired
    private SysLogininforCommonMapper logininforCommonMapper;
    @Value("${pagehelper.helperDialect}")
    private String dataSourceType;


    /**
     * 新增系统登录日志
     *
     * @param logininfor 访问日志对象
     */
    @Override
    public void insertLogininfor(SysLogininfor logininfor) {
//        logininfor.setInfoId();
        logger.error(logininfor.getUserName());
        logininfor.setBusiSeq(SequenceEnum.LOG_INFO.getCode() + DateUtils.dateTimeNow("_yyyyMM"));
        logininfor.setLoginTime(new Date());
        logininforMapper.insertLogininfor(logininfor);
    }

    /**
     * 查询系统登录日志集合
     *
     * @param logininfor 访问日志对象
     * @return 登录记录集合
     */
    @Override
    public List<SysLogininfor> selectLogininforList(SysLogininforReqDTO logininfor) {
        List<SysLogininfor> result = new ArrayList<>();
        if ("mysql".equals(dataSourceType)) {
            log.info("mysql----------"+logininfor.getPs()+"----"+logininfor.getPu());
            if (null != logininfor.getPu() && null != logininfor.getPs()) {
                Integer begin = (logininfor.getPu() - 1) * logininfor.getPs();
//            Integer end=logininfor.getPageNum()*logininfor.getPageSize();
                logininfor.setBe(begin);
                logininfor.setEn(logininfor.getPs());
            }
        }else {
            if (null != logininfor.getPu() && null != logininfor.getPs()) {
                log.info("orcle----------"+logininfor.getPs()+"----"+logininfor.getPu());
                Integer begin = (logininfor.getPu() - 1) * logininfor.getPs()+1;
                Integer end=logininfor.getPu()*logininfor.getPs();
                logininfor.setBe(begin);
                logininfor.setEn(end);
            }
        }
        log.info("+++++++++++++++++++"+logininfor.getEn()+"++++++"+logininfor.getBe()+"+++++-+-+-+-+-+-+-+-+-+-++-+-+");
        if (null != logininfor.getIds() && !logininfor.getIds().equals("")) {
            String[] middle = logininfor.getIds().split(",");
            List<String> middleids = new ArrayList<>();
            for (String mi : middle) {
                middleids.add(mi);
            }
            logininfor.setBusinessIds(middleids);
        }
        // step1 判断是否查询本月数据

        logininfor.setLoginTime("sys_logininfor_" + logininfor.getLoginTime());
        // step2 判断表是否存在 存在查询 不存在数据为空
        int line = operLogCommonMapper.judgeTable(logininfor.getLoginTime());
        if (1 == line) {
            result = logininforCommonMapper.selectLogininforList(logininfor);
            log.info("99999"+result);
        }
        return result;
    }

    /**
     * 批量删除系统登录日志
     *
     * @param infoIds 需要删除的登录日志ID
     * @return
     */
    @Override
    public int deleteLogininforByIds(Long[] infoIds) {
        return logininforMapper.deleteLogininforByIds(infoIds);
    }

    /**
     * 清空系统登录日志
     */
    @Override
    public void cleanLogininfor() {
        logininforMapper.cleanLogininfor();
    }
}
