package com.isoft.project.monitor.service.impl;

import com.isoft.common.enums.SequenceEnum;
import com.isoft.common.utils.DateUtils;
import com.isoft.common.utils.StringUtils;
import com.isoft.project.monitor.domain.SysOperLog;
import com.isoft.project.monitor.dto.SysOperLogReqDTO;
import com.isoft.project.monitor.mapper.SysOperLogCommonMapper;
import com.isoft.project.monitor.mapper.SysOperLogMapper;
import com.isoft.project.monitor.service.ISysOperLogService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

/**
 * 操作日志 服务层处理
 *
 * @author isoft
 */
@Slf4j
@Service
public class SysOperLogServiceImpl implements ISysOperLogService
{
    @Autowired
    private SysOperLogMapper operLogMapper;
    @Autowired
    private SysOperLogCommonMapper operLogCommonMapper;
    @Value("${pagehelper.helperDialect}")
    private String dataSourceType;

    /**
     * 新增操作日志
     *
     * @param operLog 操作日志对象
     */
    @Override
    public void insertOperlog(SysOperLog operLog)
    {
        operLog.setFlag("1");
        // 获取序列完整名
        operLog.setBusiSeq(SequenceEnum.OPER_LOG.getCode() + DateUtils.dateTimeNow("_yyyyMM"));
        operLogMapper.insertOperlog(operLog);
    }

    @Override
    @Transactional
    public void insertBatchOperlog(List<SysOperLog> operLog) {
        operLogMapper.insertBatchOperlog(operLog,SequenceEnum.OPER_LOG.getCode()+ DateUtils.dateTimeNow("_yyyyMM"));
    }

    /**
     * 查询系统操作日志集合
     *
     * @param operLog 操作日志对象
     * @return 操作日志集合
     */
    @Override
    public List<SysOperLog> selectOperLogList(SysOperLogReqDTO operLog)
    {
        List<SysOperLog> result = new ArrayList<>();
        if(null != operLog) {
            if ("mysql".equals(dataSourceType)) {
                log.info("mysql----------" + operLog.getPs() + "----" + operLog.getPu());
                if (null != operLog.getPu()) {
                    Integer begin = (operLog.getPu() - 1) * operLog.getPs();
                    operLog.setBe(begin);
                    operLog.setEn(operLog.getPs());
                }
            } else {
                log.info("oracle----------" + operLog.getPs() + "----" + operLog.getPu());
                if (null != operLog.getPu()) {
                    Integer begin = (operLog.getPu() - 1) * operLog.getPs() + 1;
                    Integer end = operLog.getPu() * operLog.getPs();
                    operLog.setBe(begin);
                    operLog.setEn(end);
                }
            }
            log.info("+++++++++++++++++++" + operLog.getEn() + "++++++" + operLog.getBe() + "+++++-+-+-+-+-+-+-+-+-+-++-+-+");
            if (null != operLog.getIds() && !operLog.getIds().equals("")) {
                String[] middle = operLog.getIds().split(",");
                List<String> middleids = new ArrayList<>();
                for (String mi : middle) {
                    middleids.add(mi);
                }
                operLog.setBusinessIds(middleids);
            }
            // step1 判断是否查询本月数据
            operLog.setOperTime("sys_oper_log_" + operLog.getOperTime());
            // step2 判断表是否存在 存在查询 不存在数据为空
            int line = operLogCommonMapper.judgeTable(operLog.getOperTime());
            if (1 == line) {
                result = operLogCommonMapper.selectOperLogList(operLog);
                log.info("+++666" + result);
            }
        }
        return result;
    }

    /**
     * 批量删除系统操作日志
     *
     * @param operIds 需要删除的操作日志ID
     * @return 结果
     */
    @Override
    public int deleteOperLogByIds(Long[] operIds)  {
        return operLogMapper.deleteOperLogByIds(operIds);
    }

    /**
     * 查询操作日志详细
     *
     * @param operId 操作ID
     * @return 操作日志对象
     */
    @Override
    public SysOperLog selectOperLogById(Long operId)
    {
        return operLogMapper.selectOperLogById(operId);
    }

    /**
     * 清空操作日志
     */
    @Override
    public void cleanOperLog(Long status)
    {
        operLogMapper.cleanOperLog(status);
    }
}
