package com.isoft.project.upload.service.impl;

import com.amazonaws.AmazonClientException;
import com.amazonaws.AmazonServiceException;
import com.amazonaws.services.s3.model.PutObjectResult;
import com.amazonaws.services.s3.model.S3Object;
import com.isoft.common.utils.IdUtils;
//import com.isoft.framework.feign.FeginClientUtil;
import com.isoft.project.monitor.domain.SysOperLog;
import com.isoft.project.monitor.service.ISysOperLogService;
import com.isoft.project.upload.domain.OssInfo;
import com.isoft.project.upload.enumm.BucketEnum;
import com.isoft.project.upload.enumm.FileTypeEnum;
import com.isoft.project.upload.mapper.OssInfoMapper;
import com.isoft.project.upload.service.UploadService;
import com.isoft.project.upload.util.FileFFMp4;
import com.isoft.project.upload.util.S3Util;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.web.multipart.MultipartFile;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.*;

/**
 * 参考
 * aws sdk：https://github.com/speedycloud/java-sdk
 * 例子：/src/samples/AmazonS3/S3Sample.java
 */
@Service
@Slf4j
public class UploadServiceImpl implements UploadService {

    private final OssInfoMapper ossInfoMapper;
    private final Logger logger = LoggerFactory.getLogger(UploadServiceImpl.class);

    @Value("${s3.prefix}")
    private String prefix;

    //默认 dev开发环境 （dev开发 test测试 prod生产）
    @Value("${server.type:dev}")
    private String env;

    @Value("${global.path.signImg_location}")
    private String sign;


    @Value("${global.path.base_upload_location}")
    private String base;

    @Value("${global.path.videoToMp4}")
    private String videoToMp4;
//
//    @Autowired
//    FeginClientUtil feginClientUtil;


    @Autowired
    ISysOperLogService iSysOperLogService;

    @Autowired
    public UploadServiceImpl(OssInfoMapper ossInfoMapper) {
        this.ossInfoMapper = ossInfoMapper;
    }

    /**
     * 文件夹不设置会直接传到桶下
     * 设置了上传的文件权限为 公共可读
     * 文档：https://docs.aws.amazon.com/AWSJavaSDK/latest/javadoc/overview-summary.html，页面搜索Amazon S3
     *
     * @param files 多文件
     * @param type  项目分类，0：中台，1：内勤，2：银企
     * @param dir   文件夹路径，为null上传到根目录
     * @return oss_info表主键
     */
    @Override
    public List<OssInfo> uploadSpeedy(MultipartFile[] files, int type, String dir) {
        // 桶名称
        String bucketName = BucketEnum.getNameByType(type, env);
        // 待保存
        List<OssInfo> saveList = new ArrayList<>();
        List<CompletableFuture<OssInfo>> saveCompleList = new ArrayList<>();
        long c = System.currentTimeMillis();
        // 循环文件
        for (MultipartFile file : files) {
            //开启多线程上传
            saveCompleList.add(CompletableFuture.supplyAsync(() -> {
                OssInfo ossInfo = upload(file, bucketName, dir);
                return ossInfo;
            }, pool));
        }
        CompletableFuture<OssInfo>[] array = new CompletableFuture[saveCompleList.size()];
        int i = 0;
        for (CompletableFuture<OssInfo> a : saveCompleList) {
            array[i++] = a;
        }
        CompletableFuture.allOf(array);
        saveCompleList.forEach(ossInfoCompletableFuture -> {
            try {
                if (null != ossInfoCompletableFuture.get()) {
                    saveList.add(ossInfoCompletableFuture.get());
                }
            } catch (InterruptedException e) {
                log.error(e.getMessage());
                Thread.currentThread().interrupt();
            } catch (ExecutionException e) {
                log.error(e.getMessage());
            }
        });
        long c2 = System.currentTimeMillis();
        log.info("==================文件上传耗时==" + (c2 - c) + "ms=================");
        // 如果有需要保存的对象，认为上传成功
        if (saveList.size() > 0) {
            // 调用批量上传
            log.info("===================文件上传内容=============" + saveList);
            ossInfoMapper.batchInsertOssInfo(saveList);
            // 返回所有上传的信息，去掉路径
            saveList.forEach(o -> {
                o.setBucket(null);
                o.setPath(null);
                if (!o.getPic()) {
                    o.setUrl(null);
                }
            });
            return saveList;
        } else {
            return saveList;
        }
    }

    @Override
    public List<OssInfo> uploadVideo(MultipartFile files, int type, String dir) {
        // 桶名称
        String bucketName = BucketEnum.getNameByType(type, env);
        // 待保存
        List<OssInfo> saveList = new ArrayList<>();
        List<CompletableFuture<OssInfo>> saveCompleList = new ArrayList<>();
        long c = System.currentTimeMillis();
        // 循环文件
//        for (MultipartFile file : files) {
//
//        }
        MultipartFile fileToMp4 = FileFFMp4.VideoToMp4(files, base, videoToMp4);
        //开启多线程上传
        saveCompleList.add(CompletableFuture.supplyAsync(() -> {
            OssInfo ossInfo = uploadVideo(fileToMp4, bucketName, dir);
            return ossInfo;
        }, pool));
        CompletableFuture<OssInfo>[] array = new CompletableFuture[saveCompleList.size()];
        int i = 0;
        for (CompletableFuture<OssInfo> a : saveCompleList) {
            array[i++] = a;
        }
        CompletableFuture.allOf(array);
        saveCompleList.forEach(ossInfoCompletableFuture -> {
            try {
                if (null != ossInfoCompletableFuture.get()) {
                    saveList.add(ossInfoCompletableFuture.get());
                }
            } catch (InterruptedException e) {
                log.error(e.getMessage());
                Thread.currentThread().interrupt();
            } catch (ExecutionException e) {
                log.error(e.getMessage());
            }
        });
        long c2 = System.currentTimeMillis();
        log.info("==================文件上传耗时==" + (c2 - c) + "ms=================");
        // 如果有需要保存的对象，认为上传成功
        if (saveList.size() > 0) {
            // 调用批量上传
            log.info("===================文件上传内容=============" + saveList);
            ossInfoMapper.batchInsertOssInfo(saveList);
            // 返回所有上传的信息，去掉路径
            saveList.forEach(o -> {
                o.setBucket(null);
                o.setPath(null);
                if (!o.getPic()) {
                    o.setUrl(null);
                }
            });
            return saveList;
        } else {
            return null;
        }
    }

    @Override
    public List<OssInfo> uploadRecord(MultipartFile recordFile, int type, String dir) {
        List<OssInfo> ossInfos = new ArrayList<>();
        if (recordFile != null) {
            //大于20Mb 分割文件
//            if (recordFile.getSize() > 20 * 1024 * 1024)
            String fileName = recordFile.getOriginalFilename();
            // 获取文件后缀
            String prefix = fileName.substring(fileName.lastIndexOf("."));
            try {
                File file = File.createTempFile(IdUtils.randomUUID(), prefix);
                recordFile.transferTo(file);
                //根据会议id查询该会议的当前排序
                Long s1 = System.currentTimeMillis();
                String pathName = cutFile(fileName.substring(0, fileName.lastIndexOf(".")), file, 20, 0);
                Long s2 = System.currentTimeMillis();
                log.info("--------大小" + (file.length() / (1024 * 1024)) + "Mb---音频文件---切割耗时--" + (s2 - s1) + "ms");
                //根据地址获取切割好的文件
                List<MultipartFile> multipartFileList = getFiles(pathName);
                //迅达云批量上传文件
                ossInfos = uploadSpeedy(multipartFileList, type, dir);
                Long s3 = System.currentTimeMillis();
                log.info("--------大小" + (file.length() / (1024 * 1024)) + "Mb---音频文件---上传耗时--" + (s3 - s2) + "ms");
                //删除切割文件
                File file1 = new File(pathName.substring(0, pathName.lastIndexOf("/")));
                deleteFile(file1);
                Long s4 = System.currentTimeMillis();
                log.info("--------大小" + (file.length() / (1024 * 1024)) + "Mb---音频文件---删除切割文件耗时--" + (s4 - s3) + "ms");
            } catch (IOException e) {
                log.error(e.getMessage());
            }
        }
        return ossInfos;
    }

    public List<OssInfo> uploadSpeedy(List<MultipartFile> files, int type, String dir) {
        // 桶名称
        String bucketName = BucketEnum.getNameByType(type, env);
        // 待保存
        List<OssInfo> saveList = new ArrayList<>();
        List<CompletableFuture<OssInfo>> saveCompleList = new ArrayList<>();
        long c = System.currentTimeMillis();
        // 循环文件
        for (MultipartFile file : files) {
            //开启多线程上传
            saveCompleList.add(CompletableFuture.supplyAsync(() -> {
                OssInfo ossInfo = upload(file, bucketName, dir);
                return ossInfo;
            }, pool));
        }
        CompletableFuture<OssInfo>[] array = new CompletableFuture[saveCompleList.size()];
        int i = 0;
        for (CompletableFuture<OssInfo> a : saveCompleList) {
            array[i++] = a;
        }
        CompletableFuture.allOf(array);
        saveCompleList.forEach(ossInfoCompletableFuture -> {
            try {
                if (null != ossInfoCompletableFuture.get()) {
                    saveList.add(ossInfoCompletableFuture.get());
                }
            } catch (InterruptedException e) {
                log.error(e.getMessage());
                Thread.currentThread().interrupt();
            } catch (ExecutionException e) {
                log.error(e.getMessage());
            }
        });
        long c2 = System.currentTimeMillis();
        log.info("==================文件上传耗时==" + (c2 - c) + "ms=================");
        // 如果有需要保存的对象，认为上传成功
        if (saveList.size() > 0) {
            // 调用批量上传
            log.info("===================文件上传内容=============" + saveList);
            ossInfoMapper.batchInsertOssInfo(saveList);
            return saveList;
        } else {
            return saveList;
        }
    }

    @Override
    public void downloadSpeedy(HttpServletResponse response, String id) {
        // 根据id查询数据的信息
        Long c = System.currentTimeMillis();
        OssInfo info = ossInfoMapper.selectOssInfoByPrimaryKey(id);
        Long c2 = System.currentTimeMillis();
        log.info("=========info==耗时" + (c2 - c) + "s=======" + info.toString());
        if (info != null) {
            // 调用下载
            String path = info.getPath();
            String ossKey = info.getOssKey();
            String fileName = info.getOriginalName();
            S3Object s3o = S3Util.getInstance().download(info.getBucket(), (StringUtils.isEmpty(path) ? "" : path + "/") + ossKey);
            Long c3 = System.currentTimeMillis();
            log.info("==========S3Object==耗时" + (c3 - c2) + "s=======" + s3o.toString());
            if (s3o != null) {
                // 转流下载
                try (OutputStream outputStream = response.getOutputStream()) {
                    String fileNameParam = new String(fileName.substring(fileName.lastIndexOf("/") + 1).getBytes("UTF-8"), "ISO-8859-1");
                    response.addHeader("Content-Disposition", "attachment;filename=\"" + fileNameParam + "\"");
                    outputStream.write(IOUtils.toByteArray(s3o.getObjectContent()));
                    outputStream.flush();
                } catch (IOException e) {
                    log.error(e.getMessage());
                }
            }
            Long c4 = System.currentTimeMillis();
            log.info("===========下载耗时" + (c4 - c3) + "ms=======");
        }
    }


    public void downloadSign(String id) {
        // 根据id查询数据的信息
        Long c = System.currentTimeMillis();
        String[] split = id.split(",");
        FileOutputStream sos = null;
        File dir = new File(base + sign + File.separator);
        if (!dir.exists()) {
            dir.mkdirs();
        }
        File[] subFiles = dir.listFiles();
        for (String s : Arrays.asList(split)) {
            OssInfo info = ossInfoMapper.selectOssInfoByPrimaryKey(s);
            if (info != null) {
                // 调用下载
                String path = info.getPath();
                String ossKey = info.getOssKey();
                String fileName = info.getOriginalName();
                S3Object s3o = S3Util.getInstance().download(info.getBucket(), (StringUtils.isEmpty(path) ? "" : path + "/") + ossKey);
                Long c3 = System.currentTimeMillis();

                if (s3o != null) {
                    if (subFiles != null && subFiles.length > 0) {
                        for (File subFile : subFiles) {
                            if (subFile.isFile() && fileName.contains(subFile.getName())) {
                                //删除文件
                                continue;
                            }
                        }
                    }


                    try {
                        sos = new FileOutputStream(new File(base + sign + File.separator + fileName));
                        sos.write(IOUtils.toByteArray(s3o.getObjectContent()));
                    } catch (Exception e) {
                        log.error(e.getMessage());
                    }

                }
                if (null != sos) {
                    try {
                        sos.close();
                    } catch (Exception e) {
                        log.error(e.getMessage());
                    }
                }
                // 转流下载
            }
            Long c4 = System.currentTimeMillis();
        }
    }

    @Override
    public List<OssInfo> showSpeedy(String ids) {
        List<String> idArray = Arrays.asList(ids.split(","));
        if (!CollectionUtils.isEmpty(idArray)) {
            List<OssInfo> ossInfos = ossInfoMapper.selectOssInfoByPrimaryKeys(idArray);
            List<OssInfo> lastOssInfos = new ArrayList<>();
            if(!CollectionUtils.isEmpty(ossInfos)){
                idArray.forEach(s -> {
                    ossInfos.forEach(ossInfo -> {
                        if(s.equals(ossInfo.getId())){
                            lastOssInfos.add(ossInfo);
                        }
                    });
                });
                return lastOssInfos;
            }
        }
        return new ArrayList<>();
    }

    /**
     * 下面方法判断文件类型
     * 只判断图片类型
     *
     * @param s
     * @return
     */
    private boolean checkPic(String s) {
        boolean flag = false;
        for (FileTypeEnum fileTypeEnum : FileTypeEnum.values()) {
            if (s.startsWith(fileTypeEnum.getValue())) {
                flag = true;
            }
        }
        return flag;
    }

    private String bytesToHexFileTypeString(InputStream inputStream) {
        byte[] buffer = new byte[10];
        int in = 0;
        try {
            in = inputStream.read(buffer);
        } catch (IOException e) {
            log.error(e.getMessage());
        }
        StringBuilder hexFileTypeStr = new StringBuilder();
        for (byte b : buffer) {
            String hexString = Integer.toHexString(b & 0xFF);
            if (hexString.length() < 2) {
                hexFileTypeStr.append("0");
            }
            hexFileTypeStr.append(hexString);
        }
        return hexFileTypeStr.toString();
    }

    /**
     * 文件流切割 理论支持各种文件
     *
     * @param fileName 切割文件名开头
     * @param size     切割后 每个文件大小
     * @param num      切割后 文件起始排序
     */
    public static String cutFile(String fileName, File file, int size, int num) {
        Long timestamp = System.currentTimeMillis();
        String suffix = file.getName().substring(file.getName().lastIndexOf("."));
        String pathName = "cut/" + timestamp;
        FileInputStream fis=null;
        FileOutputStream fos = null;
        try {
            Long c1 = System.currentTimeMillis();
            File file2 = new File(pathName);
            if (file2.exists() != true) {
                file2.mkdirs();
            }
            fis = new FileInputStream(file);
            int len = 0;
            int x = 0;
            int y = 1024 * 1024 * size;
            byte[] one = new byte[y];
            if (file.length() % y != 0) {
                x = (int) (file.length() / y + 1);
            } else if (file.length() % y == 0) {
                x = (int) (file.length() / y);
            }
            for (int i = 1; i <= x; i++) {
                len = fis.read(one);
                fos = new FileOutputStream(new File(file2, fileName + "-" + (i + num) + suffix));
                fos.write(one, 0, len);
                fos.close();
                fis.close();
            }
            Long c2 = System.currentTimeMillis();
            System.out.println(c2 - c1);
        } catch (Exception e) {
            log.error(e.getMessage());
        } finally {
                if (null != fos) {
                    try {
                        fos.close();
                    } catch (IOException e) {
                        log.error(e.getMessage());
                    }
                }
                if (null != fis) {
                    try{
                        fis.close();
                    } catch (IOException e) {
                        log.error(e.getMessage());
                    }
                }

        }
        return pathName;
    }


    /**
     * 方法getFiles根据指定路径获取所有的文件
     *
     * @param path
     * @return
     * @throws Exception
     */
    public List<MultipartFile> getFiles(String path) {
        //目标集合fileList
        List<MultipartFile> fileList = new ArrayList<>();
        File file = new File(path);
        InputStream inputStream = null;
        try {
            if (file.isDirectory()) {
                File[] files = file.listFiles();
                for (File fileIndex : files) {
                    //如果这个文件是目录，则进行递归搜索
                    if (fileIndex.isDirectory()) {
                        getFiles(fileIndex.getPath());
                    } else {
                        //如果文件是普通文件，则将文件句柄放入集合中
                        inputStream = new FileInputStream(fileIndex);
                        MultipartFile multipartFile = new MockMultipartFile(fileIndex.getName(), inputStream);
                        fileList.add(multipartFile);
                    }
                }
            }
        } catch (Exception e) {
            log.error(e.getMessage());
        } finally {
            try {
                if(null != inputStream) {
                    inputStream.close();
                }
            } catch (IOException e) {
                logger.error("接受异常信息：",e.getMessage(),e);
            }
        }
        return fileList;
    }


    /**
     * 先根遍历序递归删除文件夹
     *
     * @param dirFile 要被删除的文件或者目录
     * @return 删除成功返回true, 否则返回false
     */
    public static boolean deleteFile(File dirFile) {
        // 如果dir对应的文件不存在，则退出
        if (!dirFile.exists()) {
            return false;
        }

        if (dirFile.isFile()) {
            return dirFile.delete();
        } else {

            for (File file : dirFile.listFiles()) {
                deleteFile(file);
            }
        }

        return dirFile.delete();
    }

    /**
     * 上传文件
     *
     * @param file       文件
     * @param bucketName 桶
     * @param dir        上传路径
     * @return
     */
    public OssInfo upload(MultipartFile file, String bucketName, String dir) {
        // 用作 表主键 + 文件重命名
        String id = IdUtils.simpleUUID();
        // 原文件名
        String originalName = file.getOriginalFilename();
        if (StringUtils.isEmpty(originalName)) {
            originalName = file.getName();
        }
        // 后缀
        String suffix = originalName.indexOf(".") >= 0 ? originalName.substring(originalName.lastIndexOf(".")) : "";
        // 重命名文件
        String newName = id + suffix;
        // 上传
        PutObjectResult bucket;
        try {
            // 为了复用
            ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
            outputStream.write(IOUtils.toByteArray(file.getInputStream()));
            // 迅达云上传
            bucket = S3Util.getInstance().upload(bucketName, newName, new ByteArrayInputStream(outputStream.toByteArray()), dir);
            log.info("//uploadInFo========================"+bucket+"======================");
            // 不为空认为上传成功
            if (bucket != null) {
                OssInfo info = new OssInfo();
                info.setId(id);
                info.setBucket(bucketName);
                info.setPath(dir);
                info.setOssKey(newName);
                info.setMd5(bucket.getContentMd5());
                info.setEtag(bucket.getETag());
                info.setUrl(this.prefix + bucketName + "/" + (StringUtils.isEmpty(dir) ? "" : dir + "/") + newName);
                info.setFileSize(file.getSize());
                info.setFileType(suffix.replaceAll("\\.", ""));
                info.setOriginalName(originalName);
                info.setPic(this.checkPic(this.bytesToHexFileTypeString(new ByteArrayInputStream(outputStream.toByteArray()))));
                // 待保存
                return info;
            }
        } catch (IOException e) {
            log.error(e.getMessage());
            log.error(e.getMessage());
            SysOperLog sysOperLog=new SysOperLog();
            sysOperLog.setStatus(1);
            sysOperLog.setErrorMsg(e.getMessage());
            sysOperLog.setCreateTime(new Date());
            sysOperLog.setTitle("业务中台-上传功能");
            sysOperLog.setRequestMethod("UPLOAD");
            iSysOperLogService.insertOperlog(sysOperLog);

        } catch (AmazonServiceException ase) {
            log.error("Caught an AmazonServiceException, which means your request made it "
                    + "to Amazon S3, but was rejected with an error response for some reason.");
            log.error("Error Message:    " + ase.getMessage());
            log.error("HTTP Status Code: " + ase.getStatusCode());
            log.error("AWS Error Code:   " + ase.getErrorCode());
            log.error("Error Type:       " + ase.getErrorType());
            log.error("Request ID:       " + ase.getRequestId());
            SysOperLog sysOperLog=new SysOperLog();
            sysOperLog.setStatus(1);
            sysOperLog.setErrorMsg(ase.getMessage());
            sysOperLog.setCreateTime(new Date());
            sysOperLog.setTitle("业务中台-上传功能");
            sysOperLog.setRequestMethod("UPLOAD");
            iSysOperLogService.insertOperlog(sysOperLog);
        } catch (AmazonClientException ace) {
            log.error("Caught an AmazonClientException, which means the client encountered "
                    + "a serious internal problem while trying to communicate with S3, "
                    + "such as not being able to access the network.");
            log.error("Error Message: " + ace.getMessage());
            SysOperLog sysOperLog=new SysOperLog();
            sysOperLog.setStatus(1);
            sysOperLog.setErrorMsg(ace.getMessage());
            sysOperLog.setCreateTime(new Date());
            sysOperLog.setTitle("业务中台-上传功能");
            sysOperLog.setRequestMethod("UPLOAD");
            iSysOperLogService.insertOperlog(sysOperLog);
        }
        return null;
    }

    /**
     * 上传文件
     *
     * @param file       文件
     * @param bucketName 桶
     * @param dir        上传路径
     * @return
     */
    public OssInfo uploadVideo(MultipartFile file, String bucketName, String dir) {
        // 用作 表主键 + 文件重命名
        String id = IdUtils.simpleUUID();
        // 原文件名
        String originalName = file.getOriginalFilename();
        if (StringUtils.isEmpty(originalName)) {
            originalName = file.getName();
        }
        // 重命名文件
        String newName = id + ".mp4";
        // 上传
        PutObjectResult bucket;
        try {
            // 为了复用
            ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
            outputStream.write(IOUtils.toByteArray(file.getInputStream()));
            // 迅达云上传
            bucket = S3Util.getInstance().upload(bucketName, newName, new ByteArrayInputStream(outputStream.toByteArray()), dir);
            log.info("//uploadInFo========================"+bucket+"======================");
            // 不为空认为上传成功
            if (bucket != null) {
                OssInfo info = new OssInfo();
                info.setId(id);
                info.setBucket(bucketName);
                info.setPath(dir);
                info.setOssKey(newName);
                info.setMd5(bucket.getContentMd5());
                info.setEtag(bucket.getETag());
                info.setUrl(this.prefix + bucketName + "/" + (StringUtils.isEmpty(dir) ? "" : dir + "/") + newName);
                info.setFileSize(file.getSize());
                info.setFileType("video/mp4");
                info.setOriginalName(originalName);
                info.setPic(this.checkPic(this.bytesToHexFileTypeString(new ByteArrayInputStream(outputStream.toByteArray()))));
                // 待保存
                return info;
            }
        } catch (IOException e) {
            log.error(e.getMessage());
            log.error(e.getMessage());
            SysOperLog sysOperLog=new SysOperLog();
            sysOperLog.setStatus(1);
            sysOperLog.setErrorMsg(e.getMessage());
            sysOperLog.setCreateTime(new Date());
            sysOperLog.setTitle("业务中台-上传功能");
            sysOperLog.setRequestMethod("UPLOAD");
            iSysOperLogService.insertOperlog(sysOperLog);

        } catch (AmazonServiceException ase) {
            log.error("Caught an AmazonServiceException, which means your request made it "
                    + "to Amazon S3, but was rejected with an error response for some reason.");
            log.error("Error Message:    " + ase.getMessage());
            log.error("HTTP Status Code: " + ase.getStatusCode());
            log.error("AWS Error Code:   " + ase.getErrorCode());
            log.error("Error Type:       " + ase.getErrorType());
            log.error("Request ID:       " + ase.getRequestId());
            SysOperLog sysOperLog=new SysOperLog();
            sysOperLog.setStatus(1);
            sysOperLog.setErrorMsg(ase.getMessage());
            sysOperLog.setCreateTime(new Date());
            sysOperLog.setTitle("业务中台-上传功能");
            sysOperLog.setRequestMethod("UPLOAD");
            iSysOperLogService.insertOperlog(sysOperLog);
        } catch (AmazonClientException ace) {
            log.error("Caught an AmazonClientException, which means the client encountered "
                    + "a serious internal problem while trying to communicate with S3, "
                    + "such as not being able to access the network.");
            log.error("Error Message: " + ace.getMessage());
            SysOperLog sysOperLog=new SysOperLog();
            sysOperLog.setStatus(1);
            sysOperLog.setErrorMsg(ace.getMessage());
            sysOperLog.setCreateTime(new Date());
            sysOperLog.setTitle("业务中台-上传功能");
            sysOperLog.setRequestMethod("UPLOAD");
            iSysOperLogService.insertOperlog(sysOperLog);
        }
        return null;
    }

    static ThreadPoolExecutor pool = new ThreadPoolExecutor(5, 100, 10, TimeUnit.SECONDS,
            /* LinkedBlockingDeque 默认是 integer.MAX 容易内存溢出 */
            new LinkedBlockingDeque<>(100000),
            Executors.defaultThreadFactory(),
            /* 丢弃请求 策略 */
            new ThreadPoolExecutor.AbortPolicy());



    @Override
    public List<OssInfo> uploadFile(MultipartFile multipartFile, String dir, Integer flag){
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmssSS");
        String res = sdf.format(new Date());
        //设置的保存路径
        String realpath ="E://file/";
//        String realpath ="data//file/";
        //原始名称
        String originalFilename = multipartFile.getOriginalFilename();
        //生成文件名
        String newFileName = res + UUID.randomUUID().toString() + originalFilename.substring(originalFilename.lastIndexOf("."));
        //创建年月日文件夹
        Calendar date = Calendar.getInstance();
        File dateDirs = new File(date.get(Calendar.YEAR)
                + File.separator + (date.get(Calendar.MONTH) + 1) + File.separator + (date.get(Calendar.DATE)));
        //新文件
        File newFile = new File(realpath + File.separator + dateDirs + File.separator + newFileName);
        //判断目标文件所在的目录是否存在
        if (!newFile.getParentFile().exists()) {
            //如果目标文件所在的目录不存在，则创建父目录
            newFile.getParentFile().mkdirs();
        }
        System.out.println(newFile);
        //将内存中的数据写入磁盘
        try {
            multipartFile.transferTo(newFile);
        } catch (IOException e) {
            e.printStackTrace();
        }
        //完整的url
        String fileUrl = date.get(Calendar.YEAR) + "/" + (date.get(Calendar.MONTH) + 1) + "/" + (date.get(Calendar.DATE)) + "/" + newFileName;


        return null;
    }



}
