package com.isoft.project.upload.util;

import com.isoft.common.utils.IdUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.util.IOUtils;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.web.multipart.MultipartFile;
import ws.schild.jave.*;
import java.io.*;
import java.math.BigDecimal;

@Slf4j
public class FileFFMp4 {

    public static File multipartFileToFile(MultipartFile file, String base, String videoToMp4) throws Exception {
        File toFile = null;
        if (file.getSize() <= 0) {
            file = null;
        } else {
            File fileVideoToMp4 = new File(base + videoToMp4);
            //如果文件夹不存在则创建
            if (!fileVideoToMp4.exists() && !fileVideoToMp4.isDirectory()) {
                fileVideoToMp4.mkdirs();
            }
            InputStream ins = null;
            ins = file.getInputStream();
            toFile = new File(base + videoToMp4 + file.getOriginalFilename());
            inputStreamToFile(ins, toFile);
            ins.close();
        }
        return toFile;
    }

    /**
     * File 转 MultipartFile
     *
     * @param file
     * @throws Exception
     */
    public static MultipartFile fileToMultipartFile(File file) throws Exception {

        FileInputStream fileInput = new FileInputStream(file);
        MultipartFile toMultipartFile = new MockMultipartFile("file", file.getName(), "text/plain", IOUtils.toByteArray(fileInput));
        toMultipartFile.getInputStream();
        return toMultipartFile;
    }


    public static void inputStreamToFile(InputStream ins, File file) {
        OutputStream os = null;
        try {
            os = new FileOutputStream(file);
            int bytesRead = 0;
            byte[] buffer = new byte[8192];
            while ((bytesRead = ins.read(buffer, 0, 8192)) != -1) {
                os.write(buffer, 0, bytesRead);
            }
        } catch (Exception e) {
            log.error(e.getMessage());
        } finally {
            try {
                if (null != os) {
                    os.close();
                }
                if (null != ins) {
                    ins.close();
                }
            } catch (IOException e) {
                log.error(e.getMessage());
            }

        }
    }

    public static MultipartFile VideoToMp4(MultipartFile multipartFile, String base, String videoToMp4) {

        MultipartFile file = null;
        FileInputStream inputToMp4 = null;

        try {
            File fileIn = multipartFileToFile(multipartFile, base, videoToMp4);

            MultimediaObject object = new MultimediaObject(fileIn);
            String id = IdUtils.simpleUUID();

            File fileVideoToMp4 = new File(base + videoToMp4);
            //如果文件夹不存在则创建
            if (!fileVideoToMp4.exists() && !fileVideoToMp4.isDirectory()) {
                fileVideoToMp4.mkdirs();
            }

            File fileOut = new File(base + videoToMp4 + id + ".mp4");
            AudioInfo audioInfo = object.getInfo().getAudio();
            VideoInfo videoInfo = object.getInfo().getVideo();
            //设置编码的参数
            EncodingAttributes encodingAttributes = new EncodingAttributes();
            //设置音频转码的参数
            AudioAttributes audioAttributes = new AudioAttributes();
            //设置声道 编码的音频流中使用的声道数(1=单声道,2=双声道)
            audioAttributes.setChannels(2);
            //设置比特率参数除以1000就是多少kpbs
            audioAttributes.setBitRate(128000);
            //设置编码
            audioAttributes.setCodec("aac");
            //设置音量编码时候的音量值，未设置为0,如果256，则音量值不会改变
            audioAttributes.setVolume(256);
            //设置取样率44.1khz
            audioAttributes.setSamplingRate(new Integer(44100));
            //设置视频转码的参数
            VideoAttributes videoAttributes = new VideoAttributes();
            //设置编码器
            videoAttributes.setCodec("h264");
            //设置视频比特率
            videoAttributes.setBitRate(128000);
            /* 设置视频帧率,视频帧率：15 f / s  帧率越低，效果越差
             * 视频帧率（Frame rate）是用于测量显示帧数的量度。所谓的测量单位为每秒显示帧数(Frames per Second，简：FPS）或“赫兹”（Hz）
             */
            videoAttributes.setFrameRate(12);
            //设置视频大小
            videoAttributes.setSize(new VideoSize(videoInfo.getSize().getWidth(), videoInfo.getSize().getHeight()));
            System.out.println("原视频编码" + videoInfo.getDecoder());
            System.out.println("原视频高度" + videoInfo.getSize().getHeight());
            System.out.println("原视频宽度" + videoInfo.getSize().getWidth());
            System.out.println("原视频帧率" + videoInfo.getFrameRate());
            System.out.println("原视频比特率" + videoInfo.getBitRate());
            //1mb=1024kb,1kb=1024b,1mb=1024*1024
            double mb = 0.0;
            if (null != fileIn) {
                mb = Math.ceil(Double.valueOf(fileIn.length()) / (1024 * 1024));
            }
            int second = (int) object.getInfo().getDuration() / 1000;
            BigDecimal bigDecimal = new BigDecimal(String.format("%.4f", mb / second));
            System.out.println("开始压缩视频-->视频每秒平均" + bigDecimal + "MB");
            System.out.println("原音频比率" + audioInfo.getBitRate());
            System.out.println("原音频声道" + audioInfo.getChannels());
            //44.1kHz，表示每秒钟对模拟音频信号进行了44100次取样
            System.out.println("原音频取样率" + audioInfo.getSamplingRate());
            System.out.println("原音频编码" + audioInfo.getDecoder());
            long time = System.currentTimeMillis();
            encodingAttributes.setFormat("mp4");
            encodingAttributes.setAudioAttributes(audioAttributes).setVideoAttributes(videoAttributes);
            encodingAttributes.setEncodingThreads(Runtime.getRuntime().availableProcessors() / 2);
            encodingAttributes.setDecodingThreads(Runtime.getRuntime().availableProcessors() / 2);
            //encodingAttributes.setDuration(3.0f);
            Encoder encoder = new Encoder();
            encoder.encode(new MultimediaObject(fileIn), fileOut, encodingAttributes);
            System.out.println("压缩总耗时：" + (System.currentTimeMillis() - time) / 1000);
            inputToMp4 = new FileInputStream(fileOut);
            MultipartFile multipartFileToMp4 = new MockMultipartFile("file", fileOut.getName(), "text/plain", IOUtils.toByteArray(inputToMp4));
            file = multipartFileToMp4;

            inputToMp4.close();

            if (null != fileIn && fileIn.length() > 0) {
                if (!fileIn.delete()){
                    log.error("删除失败");
                }
            }

            if (null != fileOut && fileOut.length() > 0) {
                if (!fileOut.delete()){
                    log.error("删除失败");
                }
            }

        } catch (Exception e) {
        } finally {
            try {
                if (null != inputToMp4) {
                    inputToMp4.close();
                }
            } catch (IOException e) {
                e.getMessage();
            }
        }

        return file;
    }


    public static void main(String[] args) {
    }
}
