package com.isoft.project.util;

import com.alibaba.fastjson.JSONObject;
import com.aspose.words.Document;
import com.aspose.words.FontSettings;
import com.aspose.words.License;
import com.aspose.words.SaveFormat;
import com.isoft.common.utils.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class CheckRegUtil {
    private static final Logger log = LoggerFactory.getLogger(CheckRegUtil.class);

    /**
     * 校验特殊字符
     * @param str
     * @return
     */
    public static boolean isConSpeChar(String str) {
        String regEx = "[`~!@#$%^&*()+=|{}':;'\\[\\]<>/?~！@#￥%……&*——+|{}【】‘；：”“’。，、？]";
        Pattern p = Pattern.compile(regEx);
        Matcher m = p.matcher(str);
        return m.find();
    }

    /**
     * 正则校验手机号
     * @param phone 手机号
     * @return boolean
     */
    public static boolean checkPhoneReg(String phone){
        if (phone.length() != 11) {
            return false; //手机号输入格式有误，请输入11位数！
        }
        return true;
    }

    public static boolean checkPhoneReg1(String phone){
        String first = phone.substring(0,1);
        if ("1".equals(first)) {
            return true;//您的手机号" + phone + "是正确格式!
        } else {
            return false; //手机号第一位必须是1
        }
    }
//    public static boolean checkPhoneReg(String phone){
//        String regex = "^((13[0-9])|(14[5|7])|(15([0-3]|[5-9]))|(17[013678])|(18[0,5-9]))\\d{8}$";
//        if (phone.length() != 11) {
//            return false; //手机号输入格式有误，请输入11位数！
//        } else {
//            Pattern p = Pattern.compile(regex);
//            Matcher m = p.matcher(phone);
//            boolean isMatch = m.matches();
//            if (isMatch) {
//                return true; //您的手机号" + phone + "是正确格式!
//            } else {
//                return false; //您的手机号" + phone + "是错误格式！！！
//            }
//        }
//    }

    /**
     * 正则校验身份证号
     * @param idCard 身份证号
     * @return boolean
     */
    public static boolean checkIdCardReg(String idCard){
//        if (idCard.length() == 18)
//            return true;

        String reg = "(^\\d{17}(\\d|X|x)$)";
        String reg1 = "(^\\d{15}$)";
        if(idCard.matches(reg1)){
            return false;
        }
          if(!idCard.matches(reg)){
              return false;
          }
          String birthYear = idCard.substring(6,10);
          SimpleDateFormat spf = new SimpleDateFormat("yyyy");
          String nowYear = spf.format(new Date());
        return Integer.parseInt(nowYear) - Integer.parseInt(birthYear) >= 18;
//        String regularExpression = "(^[1-9]\\d{5}(18|19|20)\\d{2}((0[1-9])|(10|11|12))(([0-2][1-9])|10|20|30|31)\\d{3}[0-9Xx]$)|" +
//                "(^[1-9]\\d{5}\\d{2}((0[1-9])|(10|11|12))(([0-2][1-9])|10|20|30|31)\\d{3}$)";
//        boolean matches = idCard.matches(regularExpression);

        //判断第18位校验值
//        if (matches) {
//            if (idCard.length() == 18) {
//                try {
//                    char[] charArray = idCard.toCharArray();
//                    //前十七位加权因子
//                    int[] idCardWi = {7, 9, 10, 5, 8, 4, 2, 1, 6, 3, 7, 9, 10, 5, 8, 4, 2};
//                    //这是除以11后，可能产生的11位余数对应的验证码
//                    String[] idCardY = {"1", "0", "X", "9", "8", "7", "6", "5", "4", "3", "2"};
//                    int sum = 0;
//                    for (int i = 0; i < idCardWi.length; i++) {
//                        int current = Integer.parseInt(String.valueOf(charArray[i]));
//                        int count = current * idCardWi[i];
//                        sum += count;
//                    }
//                    char idCardLast = charArray[17];
//                    int idCardMod = sum % 11;
//                    if (idCardY[idCardMod].toUpperCase().equals(String.valueOf(idCardLast).toUpperCase())) {
//                        return true;
//                    } else {
//                        return false;
//                    }
//                } catch (Exception e) {
//                    log.error("身份证校验异常",e);
//                    return false;
//                }
//            }
//
//        }
//        return matches;

    }

    /**
     * 计算金额
     * @param money 金额
     * @param rate 利率
     * @return 计算结果
     */
    public static BigDecimal calculatedAmount(BigDecimal money, BigDecimal rate){
        if (money == null)
            return new BigDecimal("0.00");
        if (rate == null)
            return new BigDecimal("0.00");
        return money.multiply(rate).divide(new BigDecimal(100)).
                setScale(2,BigDecimal.ROUND_UP);
    }

    /**
     * 给指定日期增加天数
     * @param date 日期
     * @param day 天数
     * @return
     * @throws ParseException
     */
    public static Date addDate(Date date,long day) throws ParseException {
        long time = date.getTime(); // 得到指定日期的毫秒数
        day = day*24*60*60*1000; // 要加上的天数转换成毫秒数
        time+=day; // 相加得到新的毫秒数
        return new Date(time); // 将毫秒数转换成日期
    }

//    public static void main(String[] args) throws ParseException {
//        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss"); // 日期格式
//        Date date = dateFormat.parse("2015-07-31 15:00:00"); // 指定日期
//        Date newDate = addDate(date, 3); // 指定日期加上20天
//        System.out.println(dateFormat.format(date));// 输出格式化后的日期
//        System.out.println(dateFormat.format(newDate));
//    }

    /**
     * word 转pdf（跨平台使用）
     * @param docPath
     * @param pdfPath
     */
    public static void doc2pdf(String docPath, String pdfPath) {
        System.out.println("pdfPath = " + pdfPath);
        System.out.println("pdfPath = " + pdfPath);
        File pdfFile = new File(pdfPath);
        ByteArrayInputStream is = null;
        FileOutputStream fileOutputStream = null;
        try {
            fileOutputStream = new FileOutputStream(pdfFile);
            String s = "<License><Data><Products><Product>Aspose.Total for Java</Product><Product>Aspose.Words for Java</Product></Products><EditionType>Enterprise</EditionType><SubscriptionExpiry>20991231</SubscriptionExpiry><LicenseExpiry>20991231</LicenseExpiry><SerialNumber>8bfe198c-7f0c-4ef8-8ff0-acc3237bf0d7</SerialNumber></Data><Signature>sNLLKGMUdF0r8O1kKilWAGdgfs2BvJb/2Xp8p5iuDVfZXmhppo+d0Ran1P9TKdjV4ABwAgKXxJ3jcQTqE/2IRfqwnPf8itN8aFZlV3TJPYeD3yWE7IT55Gz6EijUpC7aKeoohTb4w2fpox58wWoF3SNp6sK6jDfiAUGEHYJ9pjU=</Signature></License>";
            is = new ByteArrayInputStream(s.getBytes());
            License license = new License();
            license.setLicense(is);
            Document doc = null;
            String os = System.getProperty("os.name");
            String path = "";
            if (os.toLowerCase().startsWith("win")) {
                path = "C:\\Windows\\Fonts";
            } else {
                path = "/usr/share/fonts/fonts";
            }
            FontSettings.getDefaultInstance().setFontsFolder(path + File.separator, true);
            doc = new Document(docPath);
            System.out.println("pdf文件：" + fileOutputStream);
            doc.save(fileOutputStream, SaveFormat.PDF);
            is.close();
            fileOutputStream.close();
        } catch (Exception e) {
            log.error("捕获异常：",e.getMessage(),e);
        } finally {
            if (is != null) {
                try {
                    is.close(); // 关闭流
                } catch (IOException e) {
                    log.error("捕获异常：",e.getMessage(),e);
                }
            }
            if (fileOutputStream != null) {
                try {
                    fileOutputStream.close(); // 关闭流
                } catch (IOException e) {
                    log.error("捕获异常：",e.getMessage(),e);
                }
            }
        }
    }

    //删除文件
    public static boolean deleFile(String path) {
        boolean flag = false;
        File file = new File(path);
        // 路径为文件且不为空则进行删除
        if (file.isFile() && file.exists())
        {
            flag = file.delete();
        }
        return flag;
    }

    /**
     * 验证邮箱合法性
     * @param email
     * @return
     */
    public static boolean isEmail(String email) {
        if (email == null)
            return false;
        String regEx1 = "^([a-z0-9A-Z]+[-|\\.]?)+[a-z0-9A-Z]@([a-z0-9A-Z]+(-[a-z0-9A-Z]+)?\\.)+[a-zA-Z]{2,}$";
        Pattern p;
        Matcher m;
        p = Pattern.compile(regEx1);
        m = p.matcher(email);
        if (m.matches())
            return true;
        else
            return false;
    }

    /**
     * 验证数字（正数）和小数点，最多保留两位小数
     * @param var
     * @return
     */
    public static boolean checkDecimal(BigDecimal var) {
        if (var.scale()>2 || var.signum() == -1) {
            return false;
        }
        return true;
    }

    /**
     * 验证不能包含数字
     * @param company
     * @return
     */
    public static boolean isContainNumber(String company) {
        Pattern p = Pattern.compile("[0-9]");
        Matcher m = p.matcher(company);
        if (m.find()) {
            return true;
        }
        return false;
    }

    /**
     * 时间转时间戳
     * @param time
     * @return
     */
    public static long timesTemp(String time){
        Date d = new Date();
        SimpleDateFormat sf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        try {
            d = sf.parse(time);// 日期转换为时间戳
        } catch (ParseException e) {
            e.printStackTrace();
        }
        long timeStemp = d.getTime();
        return timeStemp;
    }

    public static void main(String[] args) {
        boolean t = isContainNumber("duan1");
        BigDecimal var = new BigDecimal("-20.393");
        System.out.println(var.scale()+"-----"+var.signum());
        System.out.println(checkDecimal(var));
    }

    public static boolean checkError(String res) {
        JSONObject json = JSONObject.parseObject(res);
        if(StringUtils.isNotEmpty(json.getString("errorCode"))) {
            return true;
        }
        return false;
    }
}
